/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2022 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2022 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

/**
 * @brief Roles, together with users, and permissions are a fundamental concept
 * of the access controll management of CaosDB.
 *
 * @file caosdb/acm/role.h
 * @author Timm Fitchen
 * @date 2022-02-11
 */
#ifndef CAOSDB_ACM_ROLES_H
#define CAOSDB_ACM_ROLES_H

#include "caosdb/acm/permission_rule.h" // for PermissionRule
#include <memory>                       // for unique_ptr
#include <string>                       // for string

namespace caosdb::connection {
class Connection;
}

namespace caosdb::acm {

class RoleImpl;

class Role {
public:
  explicit Role(std::string name);
  explicit Role(std::unique_ptr<RoleImpl> wrapped);
  Role(std::string name, std::string description);
  Role(const Role &role);
  auto operator=(const Role &role) -> Role &;
  ~Role();
  auto ToString() const -> std::string;
  [[nodiscard]] auto GetName() const -> const std::string &;
  auto SetName(std::string name) -> void;
  [[nodiscard]] auto GetDescription() const -> const std::string &;
  auto SetDescription(std::string description) -> void;
  [[nodiscard]] auto GetPermissionRules() const -> const PermissionRules &;
  // auto SetPermissionRules(PermissionRules rules) -> void;
  auto ClearPermissionRules() -> void;
  auto Grant(const std::string &permission, bool isPriority) -> void;
  auto Deny(const std::string &permission, bool isPriority) -> void;
  auto RevokeGrant(const std::string &permission, bool isPriority) -> void;
  auto RevokeDenial(const std::string &permission, bool isPriority) -> void;
  auto AddPermissionRule(const std::string &permission, bool isGrant, bool isPriority) -> void;
  auto RemovePermissionRule(const std::string &permission, bool isGrant, bool isPriority) -> void;

  friend class caosdb::connection::Connection;

private:
  std::unique_ptr<RoleImpl> wrapped;
};

} // namespace caosdb::acm
#endif
