/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2022 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2022 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

/**
 * @brief Users, together with roles, and permissions are a fundamental concept
 * of the access controll management of CaosDB.
 *
 * @file caosdb/acm/user.h
 * @author Timm Fitchen
 * @date 2022-06-29
 */
#ifndef CAOSDB_ACM_USER_H
#define CAOSDB_ACM_USER_H

#include <memory> // for unique_ptr
#include <string> // for string

namespace caosdb::connection {
class Connection;
}

namespace caosdb::acm {

class UserImpl;

class User {
public:
  User();
  explicit User(std::string realm, std::string name);
  explicit User(std::string name);
  explicit User(std::unique_ptr<UserImpl> wrapped);
  User(const User &user);
  User(User &&user) noexcept;
  auto operator=(const User &user) -> User &;
  auto operator=(User &&user) noexcept -> User &;
  ~User();
  auto ToString() const -> std::string;
  [[nodiscard]] auto GetName() const -> const std::string &;
  auto SetName(const std::string &name) -> void;
  [[nodiscard]] auto GetRealm() const -> const std::string &;
  auto SetRealm(const std::string &realm) -> void;
  [[nodiscard]] auto GetPassword() const -> const std::string &;
  auto SetPassword(const std::string &password) -> void;

  friend class caosdb::connection::Connection;

private:
  std::unique_ptr<UserImpl> wrapped;
};

} // namespace caosdb::acm
#endif
