/*
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2022 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2022 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */
#include "caosdb/acm/role.h"
#include "caosdb/acm/v1alpha1/main.pb.h" // for ListRolesRequest
#include "caosdb/protobuf_helper.h"      // for ProtoMessageWrapper
#include <memory>                        // for unique_ptr
#include <utility>                       // for move

namespace caosdb::acm {
using caosdb::utility::ScalarProtoMessageWrapper;
using ProtoRole = caosdb::acm::v1alpha1::Role;
using ProtoRolePermissions = caosdb::acm::v1alpha1::RolePermissions;
using ProtoRoleCapabilities = caosdb::acm::v1alpha1::RoleCapabilities;
using ProtoListRoleItem = caosdb::acm::v1alpha1::ListRoleItem;

/**
 * RoleImpl class is designed to hide the implementation which makes direct use
 * of the protobuf objects underneath from the clients of the caosdb library.
 */
class RoleImpl : public ScalarProtoMessageWrapper<ProtoRole> {
public:
  /**
   * Constructor. Instanciate a role with the given name.
   */
  explicit RoleImpl(std::string name);
  /**
   * Constructor. Instanciate a role with the given name and description.
   */
  RoleImpl(std::string name, std::string description);
  /**
   * Constructor. Instanciate a role from the server's responces.
   */
  RoleImpl(ProtoRole *role);

  /**
   * Constructor. Instanciate a role from a listRoles RPC.
   */
  RoleImpl(ProtoListRoleItem &role);

  friend class Role;
  friend class caosdb::connection::Connection;

private:
  std::unique_ptr<PermissionRules> rules;
  [[nodiscard]] auto GeneratePermissionRulesSet() -> std::unique_ptr<PermissionRules>;
};

} // namespace caosdb::acm
